@@ -11,7 +11,7 @@ from docutils import nodes
 
 from ..parse import parse_names, Name, NameBreak
 from .backref import CodeExample
-from .directive import ConcatBlocksMarker, ImplicitImportMarker, AutoLinkSkipMarker
+from .directive import ConcatMarker, PrefaceMarker, SkipMarker
 
 
 @dataclass
@@ -34,22 +34,23 @@ class UserError(Exception):
 class CodeBlockAnalyser(nodes.SparseNodeVisitor):
     """Transform literal blocks of Python with links to reference documentation."""
 
-    def __init__(self, *args, source_dir: str, **kwargs):
+    def __init__(self, *args, source_dir: str, global_preface: List[str], **kwargs):
         super().__init__(*args, **kwargs)
         self.source_transforms: List[SourceTransform] = []
         relative_path = Path(self.document['source']).relative_to(source_dir)
         self.current_document = str(relative_path.with_suffix(''))
         self.title_stack = []
         self.current_refid = None
-        self.implicit_imports = []
+        self.global_preface = global_preface
+        self.prefaces = []
         self.concat_global = False
         self.concat_section = False
         self.concat_sources = []
-        self.autolink_skip = None
+        self.skip = None
 
     def unknown_visit(self, node):
         """Handle and delete custom directives, ignore others."""
-        if isinstance(node, ConcatBlocksMarker):
+        if isinstance(node, ConcatMarker):
             if node.mode not in ('off', 'section', 'on'):
                 raise UserError(
                     f'Invalid concatenation argument: `{node.mode}` '
@@ -63,21 +64,16 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
                 self.concat_section = False
                 self.concat_global = (node.mode == 'on')
             node.parent.remove(node)
-        elif isinstance(node, ImplicitImportMarker):
-            if '\n' in node.content:
-                raise UserError(
-                    'Implicit import may not contain a newline, found newline '
-                    f'in `{node.content}`, document "{self.current_document}"'
-                )
-            self.implicit_imports.append(node.content)
+        elif isinstance(node, PrefaceMarker):
+            self.prefaces.extend(node.content.split('\n'))
             node.parent.remove(node)
-        elif isinstance(node, AutoLinkSkipMarker):
+        elif isinstance(node, SkipMarker):
             if node.level not in ('next', 'section', 'file', 'off'):
                 raise UserError(
                     f'Invalid skipping argument: `{node.level}` '
                     f'in document "{self.current_document}"'
                 )
-            self.autolink_skip = node.level if node.level != 'off' else None
+            self.skip = node.level if node.level != 'off' else None
             node.parent.remove(node)
 
     def unknown_departure(self, node):
@@ -89,8 +85,8 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         if self.concat_section:
             self.concat_section = False
             self.concat_sources = []
-        if self.autolink_skip == 'section':
-            self.autolink_skip = None
+        if self.skip == 'section':
+            self.skip = None
 
     def visit_section(self, node):
         """Record first section ID."""
@@ -102,17 +98,17 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
 
     def visit_literal_block(self, node: nodes.literal_block):
         """Analyse Python code blocks."""
-        implicit_imports = self.implicit_imports
-        self.implicit_imports = []
+        prefaces = self.prefaces
+        self.prefaces = []
 
-        skip = self.autolink_skip
+        skip = self.skip
         if skip == 'next':
-            self.autolink_skip = None
+            self.skip = None
 
         if (
             len(node.children) != 1
             or not isinstance(node.children[0], nodes.Text)
-            or node.get('language', None) not in ('py', 'python')
+            or node.get('language', None) not in ('py', 'python', 'pycon')
         ):
             return
 
@@ -126,26 +122,49 @@ class CodeBlockAnalyser(nodes.SparseNodeVisitor):
         if skip:
             return
 
-        modified_source = '\n'.join(self.concat_sources + implicit_imports + [source])
+        if node['language'] == 'pycon':
+            in_statement = False
+            clean_lines = []
+            for line in source.split('\n'):
+                if line.startswith('>>> '):
+                    in_statement = True
+                    clean_lines.append(line[4:])
+                elif in_statement and line.startswith('... '):
+                    clean_lines.append(line[4:])
+                else:
+                    in_statement = False
+                    clean_lines.append('')
+            clean_source = '\n'.join(clean_lines)
+        else:
+            clean_source = source
+
+        modified_source = '\n'.join(
+            self.global_preface
+            + self.concat_sources
+            + prefaces
+            + [clean_source]
+        )
         try:
             names = parse_names(modified_source)
         except SyntaxError as e:
             msg = '\n'.join([
                 str(e) + f' in document "{self.current_document}"',
-                'Parsed source:',
+                f'Parsed source in `{node["language"]}` block:',
                 source,
             ])
             raise ParsingError(msg) from e
 
-        if implicit_imports or self.concat_sources:
+        if prefaces or self.concat_sources or self.global_preface:
             concat_lens = [s.count('\n') + 1 for s in self.concat_sources]
-            hidden_len = len(implicit_imports) + sum(concat_lens)
+            hidden_len = (
+                len(prefaces) + sum(concat_lens) + len(self.global_preface)
+            )
             for name in names:
                 name.lineno -= hidden_len
                 name.end_lineno -= hidden_len
 
         if self.concat_section or self.concat_global:
-            self.concat_sources.extend(implicit_imports + [source])
+            self.concat_sources.extend(prefaces + [clean_source])
 
         for name in names:
             if name.lineno < 1:
@@ -169,7 +188,10 @@ def link_html(
     """Inject links to code blocks on disk."""
     text = document.read_text('utf-8')
     soup = BeautifulSoup(text, 'html.parser')
-    blocks = soup.find_all('div', attrs={'class': 'highlight-python notranslate'})
+    blocks = (
+        list(soup.find_all('div', attrs={'class': 'highlight-python notranslate'}))
+        + list(soup.find_all('div', attrs={'class': 'highlight-pycon notranslate'}))
+    )
     inners = [block.select('div > pre')[0] for block in blocks]
 
     up_lvls = len(document.relative_to(out_dir).parents) - 1
@@ -178,6 +200,7 @@ def link_html(
         + '{link}" title="{title}" class="sphinx-codeautolink-a">{text}</a>'
     )
     name_pattern = '<span class="n">{name}</span>'
+    maybe_decor_pattern = '<span class="nd?">@?{name}</span>'
     period = '<span class="o">.</span>'
 
     # Expression asserts no dots before or after content nor a link after,
@@ -207,14 +230,17 @@ def link_html(
         lines = str(inner).split('\n')
 
         for name in trans.names:
-            html = period.join(
-                name_pattern.format(name=part) for part in name.code_str.split('.')
+            parts = name.code_str.split('.')
+            pattern = period.join(
+                [maybe_decor_pattern.format(name=parts[0])]
+                + [name_pattern.format(name=p) for p in parts[1:]]
             )
+
             line = lines[name.lineno - 1]
 
             # Reverse because a.b = a.b should replace from the right
             ex = call_ex if name.previous == NameBreak.call else base_ex
-            matches = list(re.finditer(ex.format(content=html), line))[::-1]
+            matches = list(re.finditer(ex.format(content=pattern), line))[::-1]
             if not matches:
                 msg = (
                     f'Could not match transformation of `{name.code_str}` '
@@ -224,12 +250,12 @@ def link_html(
                 warn(msg, RuntimeWarning)
                 continue
 
+            start, end = matches[0].span()
             link = link_pattern.format(
                 link=inventory[name.resolved_location],
                 title=name.resolved_location,
-                text=html
+                text=line[start:end]
             )
-            start, end = matches[0].span()
             lines[name.lineno - 1] = line[:start] + link + line[end:]
 
         inner.replace_with(BeautifulSoup('\n'.join(lines), 'html.parser'))
